You are viewing a plain text version of this content. The canonical link for it is here.
Posted to commits@commons.apache.org by lu...@apache.org on 2016/01/06 14:51:01 UTC

[45/50] [abbrv] [math] Field-based implementation of Adams-Moulton ODE integrator.

Field-based implementation of Adams-Moulton ODE integrator.


Project: http://git-wip-us.apache.org/repos/asf/commons-math/repo
Commit: http://git-wip-us.apache.org/repos/asf/commons-math/commit/82cf2774
Tree: http://git-wip-us.apache.org/repos/asf/commons-math/tree/82cf2774
Diff: http://git-wip-us.apache.org/repos/asf/commons-math/diff/82cf2774

Branch: refs/heads/master
Commit: 82cf2774a215ae46477e4b35decf77321e20ab34
Parents: 2a690ee
Author: Luc Maisonobe <lu...@apache.org>
Authored: Wed Jan 6 14:19:07 2016 +0100
Committer: Luc Maisonobe <lu...@apache.org>
Committed: Wed Jan 6 14:19:07 2016 +0100

----------------------------------------------------------------------
 .../math4/ode/MultistepFieldIntegrator.java     |  27 ++
 .../nonstiff/AdamsBashforthFieldIntegrator.java |  58 +--
 .../nonstiff/AdamsFieldStepInterpolator.java    |  63 ++-
 .../nonstiff/AdamsMoultonFieldIntegrator.java   | 416 +++++++++++++++++++
 .../AbstractAdamsFieldIntegratorTest.java       |   9 +-
 .../AdamsBashforthFieldIntegratorTest.java      |   6 +-
 .../nonstiff/AdamsBashforthIntegratorTest.java  |   6 +-
 .../AdamsMoultonFieldIntegratorTest.java        |  78 ++++
 8 files changed, 579 insertions(+), 84 deletions(-)
----------------------------------------------------------------------


http://git-wip-us.apache.org/repos/asf/commons-math/blob/82cf2774/src/main/java/org/apache/commons/math4/ode/MultistepFieldIntegrator.java
----------------------------------------------------------------------
diff --git a/src/main/java/org/apache/commons/math4/ode/MultistepFieldIntegrator.java b/src/main/java/org/apache/commons/math4/ode/MultistepFieldIntegrator.java
index feec974..d1ad3c8 100644
--- a/src/main/java/org/apache/commons/math4/ode/MultistepFieldIntegrator.java
+++ b/src/main/java/org/apache/commons/math4/ode/MultistepFieldIntegrator.java
@@ -316,6 +316,33 @@ public abstract class MultistepFieldIntegrator<T extends RealFieldElement<T>>
       return nSteps;
     }
 
+    /** Rescale the instance.
+     * <p>Since the scaled and Nordsieck arrays are shared with the caller,
+     * this method has the side effect of rescaling this arrays in the caller too.</p>
+     * @param newStepSize new step size to use in the scaled and Nordsieck arrays
+     */
+    protected void rescale(final T newStepSize) {
+
+        final T ratio = newStepSize.divide(getStepSize());
+        for (int i = 0; i < scaled.length; ++i) {
+            scaled[i] = scaled[i].multiply(ratio);
+        }
+
+        final T[][] nData = nordsieck.getDataRef();
+        T power = ratio;
+        for (int i = 0; i < nData.length; ++i) {
+            power = power.multiply(ratio);
+            final T[] nDataI = nData[i];
+            for (int j = 0; j < nDataI.length; ++j) {
+                nDataI[j] = nDataI[j].multiply(power);
+            }
+        }
+
+        setStepSize(newStepSize);
+
+    }
+
+
     /** Compute step grow/shrink factor according to normalized error.
      * @param error normalized error of the current step
      * @return grow/shrink factor for next step

http://git-wip-us.apache.org/repos/asf/commons-math/blob/82cf2774/src/main/java/org/apache/commons/math4/ode/nonstiff/AdamsBashforthFieldIntegrator.java
----------------------------------------------------------------------
diff --git a/src/main/java/org/apache/commons/math4/ode/nonstiff/AdamsBashforthFieldIntegrator.java b/src/main/java/org/apache/commons/math4/ode/nonstiff/AdamsBashforthFieldIntegrator.java
index db6bf4f..977573e 100644
--- a/src/main/java/org/apache/commons/math4/ode/nonstiff/AdamsBashforthFieldIntegrator.java
+++ b/src/main/java/org/apache/commons/math4/ode/nonstiff/AdamsBashforthFieldIntegrator.java
@@ -255,9 +255,11 @@ public class AdamsBashforthFieldIntegrator<T extends RealFieldElement<T>> extend
         start(equations, getStepStart(), finalTime);
 
         // reuse the step that was chosen by the starter integrator
-        AdamsFieldStepInterpolator<T> interpolator =
-                        new AdamsFieldStepInterpolator<T>(getStepSize(), getStepStart(), scaled, nordsieck,
-                                                          forward, equations.getMapper());
+        FieldODEStateAndDerivative<T> stepStart = getStepStart();
+        FieldODEStateAndDerivative<T> stepEnd   =
+                        AdamsFieldStepInterpolator.taylor(stepStart,
+                                                          stepStart.getTime().add(getStepSize()),
+                                                          getStepSize(), scaled, nordsieck);
 
         // main integration loop
         setIsLastStep(false);
@@ -270,7 +272,6 @@ public class AdamsBashforthFieldIntegrator<T extends RealFieldElement<T>> extend
             while (error.subtract(1.0).getReal() >= 0.0) {
 
                 // predict a first estimate of the state at step end
-                final FieldODEStateAndDerivative<T> stepEnd = interpolator.getCurrentState();
                 predictedY = stepEnd.getState();
 
                 // evaluate the derivative
@@ -290,26 +291,32 @@ public class AdamsBashforthFieldIntegrator<T extends RealFieldElement<T>> extend
                     // reject the step and attempt to reduce error by stepsize control
                     final T factor = computeStepGrowShrinkFactor(error);
                     rescale(filterStep(getStepSize().multiply(factor), forward, false));
-                    interpolator = new AdamsFieldStepInterpolator<T>(getStepSize(), getStepStart(), scaled, nordsieck,
-                                                                     forward, equations.getMapper());
+                    stepEnd = AdamsFieldStepInterpolator.taylor(getStepStart(),
+                                                                getStepStart().getTime().add(getStepSize()),
+                                                                getStepSize(),
+                                                                scaled,
+                                                                nordsieck);
 
                 }
             }
 
             // discrete events handling
-            System.arraycopy(predictedY, 0, y, 0, y.length);
-            setStepStart(acceptStep(interpolator, finalTime));
+            setStepStart(acceptStep(new AdamsFieldStepInterpolator<T>(getStepSize(), stepEnd,
+                                                                      predictedScaled, predictedNordsieck, forward,
+                                                                      getStepStart(), stepEnd,
+                                                                      equations.getMapper()),
+                                    finalTime));
             scaled    = predictedScaled;
             nordsieck = predictedNordsieck;
 
             if (!isLastStep()) {
 
+                System.arraycopy(predictedY, 0, y, 0, y.length);
+
                 if (resetOccurred()) {
                     // some events handler has triggered changes that
                     // invalidate the derivatives, we need to restart from scratch
                     start(equations, getStepStart(), finalTime);
-                    interpolator = new AdamsFieldStepInterpolator<T>(getStepSize(), getStepStart(), scaled, nordsieck,
-                                                                     forward, equations.getMapper());
                 }
 
                 // stepsize control for next step
@@ -330,8 +337,8 @@ public class AdamsBashforthFieldIntegrator<T extends RealFieldElement<T>> extend
                 }
 
                 rescale(hNew);
-                interpolator = new AdamsFieldStepInterpolator<T>(getStepSize(), getStepStart(), scaled, nordsieck,
-                                                                 forward, equations.getMapper());
+                stepEnd = AdamsFieldStepInterpolator.taylor(getStepStart(), getStepStart().getTime().add(getStepSize()),
+                                                            getStepSize(), scaled, nordsieck);
 
             }
 
@@ -344,31 +351,4 @@ public class AdamsBashforthFieldIntegrator<T extends RealFieldElement<T>> extend
 
     }
 
-    /** Rescale the instance.
-     * <p>Since the scaled and Nordsieck arrays are shared with the caller,
-     * this method has the side effect of rescaling this arrays in the caller too.</p>
-     * @param newStepSize new step size to use in the scaled and Nordsieck arrays
-     */
-    public void rescale(final T newStepSize) {
-
-        final T ratio = newStepSize.divide(getStepSize());
-        for (int i = 0; i < scaled.length; ++i) {
-            scaled[i] = scaled[i].multiply(ratio);
-        }
-
-        final T[][] nData = nordsieck.getDataRef();
-        T power = ratio;
-        for (int i = 0; i < nData.length; ++i) {
-            power = power.multiply(ratio);
-            final T[] nDataI = nData[i];
-            for (int j = 0; j < nDataI.length; ++j) {
-                nDataI[j] = nDataI[j].multiply(power);
-            }
-        }
-
-        setStepSize(newStepSize);
-
-    }
-
-
 }

http://git-wip-us.apache.org/repos/asf/commons-math/blob/82cf2774/src/main/java/org/apache/commons/math4/ode/nonstiff/AdamsFieldStepInterpolator.java
----------------------------------------------------------------------
diff --git a/src/main/java/org/apache/commons/math4/ode/nonstiff/AdamsFieldStepInterpolator.java b/src/main/java/org/apache/commons/math4/ode/nonstiff/AdamsFieldStepInterpolator.java
index 78c3c8e..b4b5357 100644
--- a/src/main/java/org/apache/commons/math4/ode/nonstiff/AdamsFieldStepInterpolator.java
+++ b/src/main/java/org/apache/commons/math4/ode/nonstiff/AdamsFieldStepInterpolator.java
@@ -43,6 +43,14 @@ class AdamsFieldStepInterpolator<T extends RealFieldElement<T>> extends Abstract
     /** Step size used in the first scaled derivative and Nordsieck vector. */
     private T scalingH;
 
+    /** Reference state.
+     * <p>Sometimes, the reference state is the same as globalPreviousState,
+     * sometimes it is the same as globalCurrentState, so we use a separate
+     * field to avoid any confusion.
+     * </p>
+     */
+    private final FieldODEStateAndDerivative<T> reference;
+
     /** First scaled derivative. */
     private final T[] scaled;
 
@@ -51,22 +59,7 @@ class AdamsFieldStepInterpolator<T extends RealFieldElement<T>> extends Abstract
 
     /** Simple constructor.
      * @param stepSize step size used in the scaled and Nordsieck arrays
-     * @param referenceState reference state from which Taylor expansion are estimated
-     * @param scaled first scaled derivative
-     * @param nordsieck Nordsieck vector
-     * @param isForward integration direction indicator
-     * @param equationsMapper mapper for ODE equations primary and secondary components
-     */
-    AdamsFieldStepInterpolator(final T stepSize, final FieldODEStateAndDerivative<T> referenceState,
-                               final T[] scaled, final Array2DRowFieldMatrix<T> nordsieck,
-                               final boolean isForward, final FieldEquationsMapper<T> equationsMapper) {
-        this(stepSize, scaled, nordsieck, isForward,
-             referenceState, taylor(referenceState, referenceState.getTime().add(stepSize), stepSize, scaled, nordsieck),
-             equationsMapper);
-    }
-
-    /** Simple constructor.
-     * @param stepSize step size used in the scaled and Nordsieck arrays
+     * @param reference reference state from which Taylor expansion are estimated
      * @param scaled first scaled derivative
      * @param nordsieck Nordsieck vector
      * @param isForward integration direction indicator
@@ -74,19 +67,20 @@ class AdamsFieldStepInterpolator<T extends RealFieldElement<T>> extends Abstract
      * @param globalCurrentState end of the global step
      * @param equationsMapper mapper for ODE equations primary and secondary components
      */
-    private AdamsFieldStepInterpolator(final T stepSize, final T[] scaled,
-                                       final Array2DRowFieldMatrix<T> nordsieck,
-                                       final boolean isForward,
-                                       final FieldODEStateAndDerivative<T> globalPreviousState,
-                                       final FieldODEStateAndDerivative<T> globalCurrentState,
-                                       final FieldEquationsMapper<T> equationsMapper) {
-        this(stepSize, scaled, nordsieck,
+    AdamsFieldStepInterpolator(final T stepSize, final FieldODEStateAndDerivative<T> reference,
+                               final T[] scaled, final Array2DRowFieldMatrix<T> nordsieck,
+                               final boolean isForward,
+                               final FieldODEStateAndDerivative<T> globalPreviousState,
+                               final FieldODEStateAndDerivative<T> globalCurrentState,
+                               final FieldEquationsMapper<T> equationsMapper) {
+        this(stepSize, reference, scaled, nordsieck,
              isForward, globalPreviousState, globalCurrentState,
              globalPreviousState, globalCurrentState, equationsMapper);
     }
 
     /** Simple constructor.
      * @param stepSize step size used in the scaled and Nordsieck arrays
+     * @param reference reference state from which Taylor expansion are estimated
      * @param scaled first scaled derivative
      * @param nordsieck Nordsieck vector
      * @param isForward integration direction indicator
@@ -96,8 +90,8 @@ class AdamsFieldStepInterpolator<T extends RealFieldElement<T>> extends Abstract
      * @param softCurrentState end of the restricted step
      * @param equationsMapper mapper for ODE equations primary and secondary components
      */
-    private AdamsFieldStepInterpolator(final T stepSize, final T[] scaled,
-                                       final Array2DRowFieldMatrix<T> nordsieck,
+    private AdamsFieldStepInterpolator(final T stepSize, final FieldODEStateAndDerivative<T> reference,
+                                       final T[] scaled, final Array2DRowFieldMatrix<T> nordsieck,
                                        final boolean isForward,
                                        final FieldODEStateAndDerivative<T> globalPreviousState,
                                        final FieldODEStateAndDerivative<T> globalCurrentState,
@@ -107,6 +101,7 @@ class AdamsFieldStepInterpolator<T extends RealFieldElement<T>> extends Abstract
         super(isForward, globalPreviousState, globalCurrentState,
               softPreviousState, softCurrentState, equationsMapper);
         this.scalingH  = stepSize;
+        this.reference = reference;
         this.scaled    = scaled.clone();
         this.nordsieck = new Array2DRowFieldMatrix<T>(nordsieck.getData(), false);
     }
@@ -126,7 +121,7 @@ class AdamsFieldStepInterpolator<T extends RealFieldElement<T>> extends Abstract
                                                    FieldODEStateAndDerivative<T> newSoftPreviousState,
                                                    FieldODEStateAndDerivative<T> newSoftCurrentState,
                                                    FieldEquationsMapper<T> newMapper) {
-        return new AdamsFieldStepInterpolator<T>(scalingH, scaled, nordsieck,
+        return new AdamsFieldStepInterpolator<T>(scalingH, reference, scaled, nordsieck,
                                                  newForward,
                                                  newGlobalPreviousState, newGlobalCurrentState,
                                                  newSoftPreviousState, newSoftCurrentState,
@@ -139,11 +134,11 @@ class AdamsFieldStepInterpolator<T extends RealFieldElement<T>> extends Abstract
     protected FieldODEStateAndDerivative<T> computeInterpolatedStateAndDerivatives(final FieldEquationsMapper<T> equationsMapper,
                                                                                    final T time, final T theta,
                                                                                    final T thetaH, final T oneMinusThetaH) {
-        return taylor(getPreviousState(), time, scalingH, scaled, nordsieck);
+        return taylor(reference, time, scalingH, scaled, nordsieck);
     }
 
     /** Estimate state by applying Taylor formula.
-     * @param referenceState reference state
+     * @param reference reference state
      * @param time time at which state must be estimated
      * @param stepSize step size used in the scaled and Nordsieck arrays
      * @param scaled first scaled derivative
@@ -151,12 +146,12 @@ class AdamsFieldStepInterpolator<T extends RealFieldElement<T>> extends Abstract
      * @return estimated state
      * @param <S> the type of the field elements
      */
-    private static <S extends RealFieldElement<S>> FieldODEStateAndDerivative<S> taylor(final FieldODEStateAndDerivative<S> referenceState,
-                                                                                        final S time, final S stepSize,
-                                                                                        final S[] scaled,
-                                                                                        final Array2DRowFieldMatrix<S> nordsieck) {
+    public static <S extends RealFieldElement<S>> FieldODEStateAndDerivative<S> taylor(final FieldODEStateAndDerivative<S> reference,
+                                                                                       final S time, final S stepSize,
+                                                                                       final S[] scaled,
+                                                                                       final Array2DRowFieldMatrix<S> nordsieck) {
 
-        final S x = time.subtract(referenceState.getTime());
+        final S x = time.subtract(reference.getTime());
         final S normalizedAbscissa = x.divide(stepSize);
 
         S[] stateVariation = MathArrays.buildArray(time.getField(), scaled.length);
@@ -178,7 +173,7 @@ class AdamsFieldStepInterpolator<T extends RealFieldElement<T>> extends Abstract
             }
         }
 
-        S[] estimatedState = referenceState.getState();
+        S[] estimatedState = reference.getState();
         for (int j = 0; j < stateVariation.length; ++j) {
             stateVariation[j]    = stateVariation[j].add(scaled[j].multiply(normalizedAbscissa));
             estimatedState[j] = estimatedState[j].add(stateVariation[j]);

http://git-wip-us.apache.org/repos/asf/commons-math/blob/82cf2774/src/main/java/org/apache/commons/math4/ode/nonstiff/AdamsMoultonFieldIntegrator.java
----------------------------------------------------------------------
diff --git a/src/main/java/org/apache/commons/math4/ode/nonstiff/AdamsMoultonFieldIntegrator.java b/src/main/java/org/apache/commons/math4/ode/nonstiff/AdamsMoultonFieldIntegrator.java
new file mode 100644
index 0000000..b09942d
--- /dev/null
+++ b/src/main/java/org/apache/commons/math4/ode/nonstiff/AdamsMoultonFieldIntegrator.java
@@ -0,0 +1,416 @@
+/*
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
+ *
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
+
+package org.apache.commons.math4.ode.nonstiff;
+
+import java.util.Arrays;
+
+import org.apache.commons.math4.Field;
+import org.apache.commons.math4.RealFieldElement;
+import org.apache.commons.math4.exception.DimensionMismatchException;
+import org.apache.commons.math4.exception.MaxCountExceededException;
+import org.apache.commons.math4.exception.NoBracketingException;
+import org.apache.commons.math4.exception.NumberIsTooSmallException;
+import org.apache.commons.math4.linear.Array2DRowFieldMatrix;
+import org.apache.commons.math4.linear.FieldMatrixPreservingVisitor;
+import org.apache.commons.math4.ode.FieldExpandableODE;
+import org.apache.commons.math4.ode.FieldODEState;
+import org.apache.commons.math4.ode.FieldODEStateAndDerivative;
+import org.apache.commons.math4.util.MathArrays;
+import org.apache.commons.math4.util.MathUtils;
+
+
+/**
+ * This class implements implicit Adams-Moulton integrators for Ordinary
+ * Differential Equations.
+ *
+ * <p>Adams-Moulton methods (in fact due to Adams alone) are implicit
+ * multistep ODE solvers. This implementation is a variation of the classical
+ * one: it uses adaptive stepsize to implement error control, whereas
+ * classical implementations are fixed step size. The value of state vector
+ * at step n+1 is a simple combination of the value at step n and of the
+ * derivatives at steps n+1, n, n-1 ... Since y'<sub>n+1</sub> is needed to
+ * compute y<sub>n+1</sub>, another method must be used to compute a first
+ * estimate of y<sub>n+1</sub>, then compute y'<sub>n+1</sub>, then compute
+ * a final estimate of y<sub>n+1</sub> using the following formulas. Depending
+ * on the number k of previous steps one wants to use for computing the next
+ * value, different formulas are available for the final estimate:</p>
+ * <ul>
+ *   <li>k = 1: y<sub>n+1</sub> = y<sub>n</sub> + h y'<sub>n+1</sub></li>
+ *   <li>k = 2: y<sub>n+1</sub> = y<sub>n</sub> + h (y'<sub>n+1</sub>+y'<sub>n</sub>)/2</li>
+ *   <li>k = 3: y<sub>n+1</sub> = y<sub>n</sub> + h (5y'<sub>n+1</sub>+8y'<sub>n</sub>-y'<sub>n-1</sub>)/12</li>
+ *   <li>k = 4: y<sub>n+1</sub> = y<sub>n</sub> + h (9y'<sub>n+1</sub>+19y'<sub>n</sub>-5y'<sub>n-1</sub>+y'<sub>n-2</sub>)/24</li>
+ *   <li>...</li>
+ * </ul>
+ *
+ * <p>A k-steps Adams-Moulton method is of order k+1.</p>
+ *
+ * <h3>Implementation details</h3>
+ *
+ * <p>We define scaled derivatives s<sub>i</sub>(n) at step n as:
+ * <pre>
+ * s<sub>1</sub>(n) = h y'<sub>n</sub> for first derivative
+ * s<sub>2</sub>(n) = h<sup>2</sup>/2 y''<sub>n</sub> for second derivative
+ * s<sub>3</sub>(n) = h<sup>3</sup>/6 y'''<sub>n</sub> for third derivative
+ * ...
+ * s<sub>k</sub>(n) = h<sup>k</sup>/k! y<sup>(k)</sup><sub>n</sub> for k<sup>th</sup> derivative
+ * </pre></p>
+ *
+ * <p>The definitions above use the classical representation with several previous first
+ * derivatives. Lets define
+ * <pre>
+ *   q<sub>n</sub> = [ s<sub>1</sub>(n-1) s<sub>1</sub>(n-2) ... s<sub>1</sub>(n-(k-1)) ]<sup>T</sup>
+ * </pre>
+ * (we omit the k index in the notation for clarity). With these definitions,
+ * Adams-Moulton methods can be written:
+ * <ul>
+ *   <li>k = 1: y<sub>n+1</sub> = y<sub>n</sub> + s<sub>1</sub>(n+1)</li>
+ *   <li>k = 2: y<sub>n+1</sub> = y<sub>n</sub> + 1/2 s<sub>1</sub>(n+1) + [ 1/2 ] q<sub>n+1</sub></li>
+ *   <li>k = 3: y<sub>n+1</sub> = y<sub>n</sub> + 5/12 s<sub>1</sub>(n+1) + [ 8/12 -1/12 ] q<sub>n+1</sub></li>
+ *   <li>k = 4: y<sub>n+1</sub> = y<sub>n</sub> + 9/24 s<sub>1</sub>(n+1) + [ 19/24 -5/24 1/24 ] q<sub>n+1</sub></li>
+ *   <li>...</li>
+ * </ul></p>
+ *
+ * <p>Instead of using the classical representation with first derivatives only (y<sub>n</sub>,
+ * s<sub>1</sub>(n+1) and q<sub>n+1</sub>), our implementation uses the Nordsieck vector with
+ * higher degrees scaled derivatives all taken at the same step (y<sub>n</sub>, s<sub>1</sub>(n)
+ * and r<sub>n</sub>) where r<sub>n</sub> is defined as:
+ * <pre>
+ * r<sub>n</sub> = [ s<sub>2</sub>(n), s<sub>3</sub>(n) ... s<sub>k</sub>(n) ]<sup>T</sup>
+ * </pre>
+ * (here again we omit the k index in the notation for clarity)
+ * </p>
+ *
+ * <p>Taylor series formulas show that for any index offset i, s<sub>1</sub>(n-i) can be
+ * computed from s<sub>1</sub>(n), s<sub>2</sub>(n) ... s<sub>k</sub>(n), the formula being exact
+ * for degree k polynomials.
+ * <pre>
+ * s<sub>1</sub>(n-i) = s<sub>1</sub>(n) + &sum;<sub>j&gt;0</sub> (j+1) (-i)<sup>j</sup> s<sub>j+1</sub>(n)
+ * </pre>
+ * The previous formula can be used with several values for i to compute the transform between
+ * classical representation and Nordsieck vector. The transform between r<sub>n</sub>
+ * and q<sub>n</sub> resulting from the Taylor series formulas above is:
+ * <pre>
+ * q<sub>n</sub> = s<sub>1</sub>(n) u + P r<sub>n</sub>
+ * </pre>
+ * where u is the [ 1 1 ... 1 ]<sup>T</sup> vector and P is the (k-1)&times;(k-1) matrix built
+ * with the (j+1) (-i)<sup>j</sup> terms with i being the row number starting from 1 and j being
+ * the column number starting from 1:
+ * <pre>
+ *        [  -2   3   -4    5  ... ]
+ *        [  -4  12  -32   80  ... ]
+ *   P =  [  -6  27 -108  405  ... ]
+ *        [  -8  48 -256 1280  ... ]
+ *        [          ...           ]
+ * </pre></p>
+ *
+ * <p>Using the Nordsieck vector has several advantages:
+ * <ul>
+ *   <li>it greatly simplifies step interpolation as the interpolator mainly applies
+ *   Taylor series formulas,</li>
+ *   <li>it simplifies step changes that occur when discrete events that truncate
+ *   the step are triggered,</li>
+ *   <li>it allows to extend the methods in order to support adaptive stepsize.</li>
+ * </ul></p>
+ *
+ * <p>The predicted Nordsieck vector at step n+1 is computed from the Nordsieck vector at step
+ * n as follows:
+ * <ul>
+ *   <li>Y<sub>n+1</sub> = y<sub>n</sub> + s<sub>1</sub>(n) + u<sup>T</sup> r<sub>n</sub></li>
+ *   <li>S<sub>1</sub>(n+1) = h f(t<sub>n+1</sub>, Y<sub>n+1</sub>)</li>
+ *   <li>R<sub>n+1</sub> = (s<sub>1</sub>(n) - S<sub>1</sub>(n+1)) P<sup>-1</sup> u + P<sup>-1</sup> A P r<sub>n</sub></li>
+ * </ul>
+ * where A is a rows shifting matrix (the lower left part is an identity matrix):
+ * <pre>
+ *        [ 0 0   ...  0 0 | 0 ]
+ *        [ ---------------+---]
+ *        [ 1 0   ...  0 0 | 0 ]
+ *    A = [ 0 1   ...  0 0 | 0 ]
+ *        [       ...      | 0 ]
+ *        [ 0 0   ...  1 0 | 0 ]
+ *        [ 0 0   ...  0 1 | 0 ]
+ * </pre>
+ * From this predicted vector, the corrected vector is computed as follows:
+ * <ul>
+ *   <li>y<sub>n+1</sub> = y<sub>n</sub> + S<sub>1</sub>(n+1) + [ -1 +1 -1 +1 ... &plusmn;1 ] r<sub>n+1</sub></li>
+ *   <li>s<sub>1</sub>(n+1) = h f(t<sub>n+1</sub>, y<sub>n+1</sub>)</li>
+ *   <li>r<sub>n+1</sub> = R<sub>n+1</sub> + (s<sub>1</sub>(n+1) - S<sub>1</sub>(n+1)) P<sup>-1</sup> u</li>
+ * </ul>
+ * where the upper case Y<sub>n+1</sub>, S<sub>1</sub>(n+1) and R<sub>n+1</sub> represent the
+ * predicted states whereas the lower case y<sub>n+1</sub>, s<sub>n+1</sub> and r<sub>n+1</sub>
+ * represent the corrected states.</p>
+ *
+ * <p>The P<sup>-1</sup>u vector and the P<sup>-1</sup> A P matrix do not depend on the state,
+ * they only depend on k and therefore are precomputed once for all.</p>
+ *
+ * @param <T> the type of the field elements
+ * @since 3.6
+ */
+public class AdamsMoultonFieldIntegrator<T extends RealFieldElement<T>> extends AdamsFieldIntegrator<T> {
+
+    /** Integrator method name. */
+    private static final String METHOD_NAME = "Adams-Moulton";
+
+    /**
+     * Build an Adams-Moulton integrator with the given order and error control parameters.
+     * @param field field to which the time and state vector elements belong
+     * @param nSteps number of steps of the method excluding the one being computed
+     * @param minStep minimal step (sign is irrelevant, regardless of
+     * integration direction, forward or backward), the last step can
+     * be smaller than this
+     * @param maxStep maximal step (sign is irrelevant, regardless of
+     * integration direction, forward or backward), the last step can
+     * be smaller than this
+     * @param scalAbsoluteTolerance allowed absolute error
+     * @param scalRelativeTolerance allowed relative error
+     * @exception NumberIsTooSmallException if order is 1 or less
+     */
+    public AdamsMoultonFieldIntegrator(final Field<T> field, final int nSteps,
+                                       final double minStep, final double maxStep,
+                                       final double scalAbsoluteTolerance,
+                                       final double scalRelativeTolerance)
+        throws NumberIsTooSmallException {
+        super(field, METHOD_NAME, nSteps, nSteps + 1, minStep, maxStep,
+              scalAbsoluteTolerance, scalRelativeTolerance);
+    }
+
+    /**
+     * Build an Adams-Moulton integrator with the given order and error control parameters.
+     * @param field field to which the time and state vector elements belong
+     * @param nSteps number of steps of the method excluding the one being computed
+     * @param minStep minimal step (sign is irrelevant, regardless of
+     * integration direction, forward or backward), the last step can
+     * be smaller than this
+     * @param maxStep maximal step (sign is irrelevant, regardless of
+     * integration direction, forward or backward), the last step can
+     * be smaller than this
+     * @param vecAbsoluteTolerance allowed absolute error
+     * @param vecRelativeTolerance allowed relative error
+     * @exception IllegalArgumentException if order is 1 or less
+     */
+    public AdamsMoultonFieldIntegrator(final Field<T> field, final int nSteps,
+                                       final double minStep, final double maxStep,
+                                       final double[] vecAbsoluteTolerance,
+                                       final double[] vecRelativeTolerance)
+        throws IllegalArgumentException {
+        super(field, METHOD_NAME, nSteps, nSteps + 1, minStep, maxStep,
+              vecAbsoluteTolerance, vecRelativeTolerance);
+    }
+
+    /** {@inheritDoc} */
+    @Override
+    public FieldODEStateAndDerivative<T> integrate(final FieldExpandableODE<T> equations,
+                                                   final FieldODEState<T> initialState,
+                                                   final T finalTime)
+        throws NumberIsTooSmallException, DimensionMismatchException,
+               MaxCountExceededException, NoBracketingException {
+
+        sanityChecks(initialState, finalTime);
+        final T   t0 = initialState.getTime();
+        final T[] y  = equations.getMapper().mapState(initialState);
+        setStepStart(initIntegration(equations, t0, y, finalTime));
+        final boolean forward = finalTime.subtract(initialState.getTime()).getReal() > 0;
+
+        // compute the initial Nordsieck vector using the configured starter integrator
+        start(equations, getStepStart(), finalTime);
+
+        // reuse the step that was chosen by the starter integrator
+        FieldODEStateAndDerivative<T> stepStart = getStepStart();
+        FieldODEStateAndDerivative<T> stepEnd   =
+                        AdamsFieldStepInterpolator.taylor(stepStart,
+                                                          stepStart.getTime().add(getStepSize()),
+                                                          getStepSize(), scaled, nordsieck);
+
+        // main integration loop
+        setIsLastStep(false);
+        do {
+
+            T[] predictedY = null;
+            final T[] predictedScaled = MathArrays.buildArray(getField(), y.length);
+            Array2DRowFieldMatrix<T> predictedNordsieck = null;
+            T error = getField().getZero().add(10);
+            while (error.subtract(1.0).getReal() >= 0.0) {
+
+                // predict a first estimate of the state at step end (P in the PECE sequence)
+                predictedY = stepEnd.getState();
+
+                // evaluate a first estimate of the derivative (first E in the PECE sequence)
+                final T[] yDot = computeDerivatives(stepEnd.getTime(), predictedY);
+
+                // update Nordsieck vector
+                for (int j = 0; j < predictedScaled.length; ++j) {
+                    predictedScaled[j] = getStepSize().multiply(yDot[j]);
+                }
+                predictedNordsieck = updateHighOrderDerivativesPhase1(nordsieck);
+                updateHighOrderDerivativesPhase2(scaled, predictedScaled, predictedNordsieck);
+
+                // apply correction (C in the PECE sequence)
+                error = predictedNordsieck.walkInOptimizedOrder(new Corrector(y, predictedScaled, predictedY));
+
+                if (error.subtract(1.0).getReal() >= 0.0) {
+                    // reject the step and attempt to reduce error by stepsize control
+                    final T factor = computeStepGrowShrinkFactor(error);
+                    rescale(filterStep(getStepSize().multiply(factor), forward, false));
+                    stepEnd = AdamsFieldStepInterpolator.taylor(getStepStart(),
+                                                                getStepStart().getTime().add(getStepSize()),
+                                                                getStepSize(),
+                                                                scaled,
+                                                                nordsieck);
+                }
+            }
+
+            // evaluate a final estimate of the derivative (second E in the PECE sequence)
+            final T[] correctedYDot = computeDerivatives(stepEnd.getTime(), predictedY);
+
+            // update Nordsieck vector
+            final T[] correctedScaled = MathArrays.buildArray(getField(), y.length);
+            for (int j = 0; j < correctedScaled.length; ++j) {
+                correctedScaled[j] = getStepSize().multiply(correctedYDot[j]);
+            }
+            updateHighOrderDerivativesPhase2(predictedScaled, correctedScaled, predictedNordsieck);
+
+            // discrete events handling
+            stepEnd = new FieldODEStateAndDerivative<T>(stepEnd.getTime(), predictedY, correctedYDot);
+            setStepStart(acceptStep(new AdamsFieldStepInterpolator<T>(getStepSize(), stepEnd,
+                                                                      correctedScaled, predictedNordsieck, forward,
+                                                                      getStepStart(), stepEnd,
+                                                                      equations.getMapper()),
+                                    finalTime));
+            scaled    = correctedScaled;
+            nordsieck = predictedNordsieck;
+
+            if (!isLastStep()) {
+
+                System.arraycopy(predictedY, 0, y, 0, y.length);
+
+                if (resetOccurred()) {
+                    // some events handler has triggered changes that
+                    // invalidate the derivatives, we need to restart from scratch
+                    start(equations, getStepStart(), finalTime);
+                }
+
+                // stepsize control for next step
+                final T  factor     = computeStepGrowShrinkFactor(error);
+                final T  scaledH    = getStepSize().multiply(factor);
+                final T  nextT      = getStepStart().getTime().add(scaledH);
+                final boolean nextIsLast = forward ?
+                                           nextT.subtract(finalTime).getReal() >= 0 :
+                                           nextT.subtract(finalTime).getReal() <= 0;
+                T hNew = filterStep(scaledH, forward, nextIsLast);
+
+                final T  filteredNextT      = getStepStart().getTime().add(hNew);
+                final boolean filteredNextIsLast = forward ?
+                                                   filteredNextT.subtract(finalTime).getReal() >= 0 :
+                                                   filteredNextT.subtract(finalTime).getReal() <= 0;
+                if (filteredNextIsLast) {
+                    hNew = finalTime.subtract(getStepStart().getTime());
+                }
+
+                rescale(hNew);
+                stepEnd = AdamsFieldStepInterpolator.taylor(getStepStart(), getStepStart().getTime().add(getStepSize()),
+                                                            getStepSize(), scaled, nordsieck);
+
+            }
+
+        } while (!isLastStep());
+
+        final FieldODEStateAndDerivative<T> finalState = getStepStart();
+        setStepStart(null);
+        setStepSize(null);
+        return finalState;
+
+    }
+
+    /** Corrector for current state in Adams-Moulton method.
+     * <p>
+     * This visitor implements the Taylor series formula:
+     * <pre>
+     * Y<sub>n+1</sub> = y<sub>n</sub> + s<sub>1</sub>(n+1) + [ -1 +1 -1 +1 ... &plusmn;1 ] r<sub>n+1</sub>
+     * </pre>
+     * </p>
+     */
+    private class Corrector implements FieldMatrixPreservingVisitor<T> {
+
+        /** Previous state. */
+        private final T[] previous;
+
+        /** Current scaled first derivative. */
+        private final T[] scaled;
+
+        /** Current state before correction. */
+        private final T[] before;
+
+        /** Current state after correction. */
+        private final T[] after;
+
+        /** Simple constructor.
+         * @param previous previous state
+         * @param scaled current scaled first derivative
+         * @param state state to correct (will be overwritten after visit)
+         */
+        Corrector(final T[] previous, final T[] scaled, final T[] state) {
+            this.previous = previous;
+            this.scaled   = scaled;
+            this.after    = state;
+            this.before   = state.clone();
+        }
+
+        /** {@inheritDoc} */
+        public void start(int rows, int columns,
+                          int startRow, int endRow, int startColumn, int endColumn) {
+            Arrays.fill(after, getField().getZero());
+        }
+
+        /** {@inheritDoc} */
+        public void visit(int row, int column, T value) {
+            if ((row & 0x1) == 0) {
+                after[column] = after[column].subtract(value);
+            } else {
+                after[column] = after[column].add(value);
+            }
+        }
+
+        /**
+         * End visiting the Nordsieck vector.
+         * <p>The correction is used to control stepsize. So its amplitude is
+         * considered to be an error, which must be normalized according to
+         * error control settings. If the normalized value is greater than 1,
+         * the correction was too large and the step must be rejected.</p>
+         * @return the normalized correction, if greater than 1, the step
+         * must be rejected
+         */
+        public T end() {
+
+            T error = getField().getZero();
+            for (int i = 0; i < after.length; ++i) {
+                after[i] = after[i].add(previous[i].add(scaled[i]));
+                if (i < mainSetDimension) {
+                    final T yScale = MathUtils.max(previous[i].abs(), after[i].abs());
+                    final T tol = (vecAbsoluteTolerance == null) ?
+                                  yScale.multiply(scalRelativeTolerance).add(scalAbsoluteTolerance) :
+                                  yScale.multiply(vecRelativeTolerance[i]).add(vecAbsoluteTolerance[i]);
+                    final T ratio  = after[i].subtract(before[i]).divide(tol); // (corrected-predicted)/tol
+                    error = error.add(ratio.multiply(ratio));
+                }
+            }
+
+            return error.divide(mainSetDimension).sqrt();
+
+        }
+    }
+
+}

http://git-wip-us.apache.org/repos/asf/commons-math/blob/82cf2774/src/test/java/org/apache/commons/math4/ode/nonstiff/AbstractAdamsFieldIntegratorTest.java
----------------------------------------------------------------------
diff --git a/src/test/java/org/apache/commons/math4/ode/nonstiff/AbstractAdamsFieldIntegratorTest.java b/src/test/java/org/apache/commons/math4/ode/nonstiff/AbstractAdamsFieldIntegratorTest.java
index 0d22831..606c9e8 100644
--- a/src/test/java/org/apache/commons/math4/ode/nonstiff/AbstractAdamsFieldIntegratorTest.java
+++ b/src/test/java/org/apache/commons/math4/ode/nonstiff/AbstractAdamsFieldIntegratorTest.java
@@ -74,10 +74,10 @@ public abstract class AbstractAdamsFieldIntegratorTest {
     public abstract void testIncreasingTolerance();
 
     protected <T extends RealFieldElement<T>> void doTestIncreasingTolerance(final Field<T> field,
-                                                                             int ratioMin, int ratioMax) {
+                                                                             double ratioMin, double ratioMax) {
 
         int previousCalls = Integer.MAX_VALUE;
-        for (int i = -12; i < -5; ++i) {
+        for (int i = -12; i < -2; ++i) {
             TestFieldProblem1<T> pb = new TestFieldProblem1<T>(field);
             double minStep = 0;
             double maxStep = pb.getFinalTime().subtract(pb.getInitialState().getTime()).getReal();
@@ -106,7 +106,7 @@ public abstract class AbstractAdamsFieldIntegratorTest {
     @Test(expected = MaxCountExceededException.class)
     public abstract void exceedMaxEvaluations();
 
-    protected <T extends RealFieldElement<T>> void doExceedMaxEvaluations(final Field<T> field) {
+    protected <T extends RealFieldElement<T>> void doExceedMaxEvaluations(final Field<T> field, final int max) {
 
         TestFieldProblem1<T> pb  = new TestFieldProblem1<T>(field);
         double range = pb.getFinalTime().subtract(pb.getInitialState().getTime()).getReal();
@@ -114,7 +114,7 @@ public abstract class AbstractAdamsFieldIntegratorTest {
         FirstOrderFieldIntegrator<T> integ = createIntegrator(field, 2, 0, range, 1.0e-12, 1.0e-12);
         TestFieldProblemHandler<T> handler = new TestFieldProblemHandler<T>(pb, integ);
         integ.addStepHandler(handler);
-        integ.setMaxEvaluations(650);
+        integ.setMaxEvaluations(max);
         integ.integrate(new FieldExpandableODE<T>(pb), pb.getInitialState(), pb.getFinalTime());
 
     }
@@ -132,7 +132,6 @@ public abstract class AbstractAdamsFieldIntegratorTest {
         double range = pb.getFinalTime().subtract(pb.getInitialState().getTime()).getReal();
 
         AdamsFieldIntegrator<T> integ = createIntegrator(field, 4, 0, range, 1.0e-12, 1.0e-12);
-        integ.setStarterIntegrator(new PerfectStarter<T>(pb, (integ.getNSteps() + 5) / 2));
         TestFieldProblemHandler<T> handler = new TestFieldProblemHandler<T>(pb, integ);
         integ.addStepHandler(handler);
         integ.integrate(new FieldExpandableODE<T>(pb), pb.getInitialState(), pb.getFinalTime());

http://git-wip-us.apache.org/repos/asf/commons-math/blob/82cf2774/src/test/java/org/apache/commons/math4/ode/nonstiff/AdamsBashforthFieldIntegratorTest.java
----------------------------------------------------------------------
diff --git a/src/test/java/org/apache/commons/math4/ode/nonstiff/AdamsBashforthIntegratorTest.java b/src/test/java/org/apache/commons/math4/ode/nonstiff/AdamsBashforthIntegratorTest.java
index 85c7e43..f655238 100644
--- a/src/test/java/org/apache/commons/math4/ode/nonstiff/AdamsBashforthIntegratorTest.java
+++ b/src/test/java/org/apache/commons/math4/ode/nonstiff/AdamsBashforthIntegratorTest.java
@@ -77,7 +77,7 @@ public class AdamsBashforthIntegratorTest {
     public void testIncreasingTolerance() throws DimensionMismatchException, NumberIsTooSmallException, MaxCountExceededException, NoBracketingException {
 
         int previousCalls = Integer.MAX_VALUE;
-        for (int i = -12; i < -5; ++i) {
+        for (int i = -12; i < -2; ++i) {
             TestProblem1 pb = new TestProblem1();
             double minStep = 0;
             double maxStep = pb.getFinalTime() - pb.getInitialTime();
@@ -93,10 +93,10 @@ public class AdamsBashforthIntegratorTest {
                             pb.getInitialTime(), pb.getInitialState(),
                             pb.getFinalTime(), new double[pb.getDimension()]);
 
-            // the 8 and 122 factors are only valid for this test
+            // the 2.6 and 122 factors are only valid for this test
             // and has been obtained from trial and error
             // there are no general relationship between local and global errors
-            Assert.assertTrue(handler.getMaximalValueError() > (  8 * scalAbsoluteTolerance));
+            Assert.assertTrue(handler.getMaximalValueError() > (2.6 * scalAbsoluteTolerance));
             Assert.assertTrue(handler.getMaximalValueError() < (122 * scalAbsoluteTolerance));
 
             int calls = pb.getCalls();

http://git-wip-us.apache.org/repos/asf/commons-math/blob/82cf2774/src/test/java/org/apache/commons/math4/ode/nonstiff/AdamsMoultonFieldIntegratorTest.java
----------------------------------------------------------------------
diff --git a/src/test/java/org/apache/commons/math4/ode/nonstiff/AdamsMoultonFieldIntegratorTest.java b/src/test/java/org/apache/commons/math4/ode/nonstiff/AdamsMoultonFieldIntegratorTest.java
new file mode 100644
index 0000000..c44124a
--- /dev/null
+++ b/src/test/java/org/apache/commons/math4/ode/nonstiff/AdamsMoultonFieldIntegratorTest.java
@@ -0,0 +1,78 @@
+/*
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
+ *
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
+
+package org.apache.commons.math4.ode.nonstiff;
+
+
+import org.apache.commons.math4.Field;
+import org.apache.commons.math4.RealFieldElement;
+import org.apache.commons.math4.exception.MathIllegalStateException;
+import org.apache.commons.math4.exception.MaxCountExceededException;
+import org.apache.commons.math4.exception.NumberIsTooSmallException;
+import org.apache.commons.math4.util.Decimal64Field;
+import org.junit.Test;
+
+public class AdamsMoultonFieldIntegratorTest extends AbstractAdamsFieldIntegratorTest {
+
+    protected <T extends RealFieldElement<T>> AdamsFieldIntegrator<T>
+    createIntegrator(Field<T> field, final int nSteps, final double minStep, final double maxStep,
+                     final double scalAbsoluteTolerance, final double scalRelativeTolerance) {
+        return new AdamsMoultonFieldIntegrator<T>(field, nSteps, minStep, maxStep,
+                        scalAbsoluteTolerance, scalRelativeTolerance);
+    }
+
+    protected <T extends RealFieldElement<T>> AdamsFieldIntegrator<T>
+    createIntegrator(Field<T> field, final int nSteps, final double minStep, final double maxStep,
+                     final double[] vecAbsoluteTolerance, final double[] vecRelativeTolerance) {
+        return new AdamsMoultonFieldIntegrator<T>(field, nSteps, minStep, maxStep,
+                        vecAbsoluteTolerance, vecRelativeTolerance);
+    }
+
+    @Test(expected=NumberIsTooSmallException.class)
+    public void testMinStep() {
+        doDimensionCheck(Decimal64Field.getInstance());
+    }
+
+    @Test
+    public void testIncreasingTolerance() {
+        // the 0.45 and 8.69 factors are only valid for this test
+        // and has been obtained from trial and error
+        // there are no general relationship between local and global errors
+        doTestIncreasingTolerance(Decimal64Field.getInstance(), 0.45, 8.69);
+    }
+
+    @Test(expected = MaxCountExceededException.class)
+    public void exceedMaxEvaluations() {
+        doExceedMaxEvaluations(Decimal64Field.getInstance(), 650);
+    }
+
+    @Test
+    public void backward() {
+        doBackward(Decimal64Field.getInstance(), 3.0e-9, 3.0e-9, 1.0e-16, "Adams-Moulton");
+    }
+
+    @Test
+    public void polynomial() {
+        doPolynomial(Decimal64Field.getInstance(), 5, 2.2e-05, 1.1e-11);
+    }
+
+    @Test(expected=MathIllegalStateException.class)
+    public void testStartFailure() {
+        doTestStartFailure(Decimal64Field.getInstance());
+    }
+
+}
diff --git a/src/test/java/org/apache/commons/math4/ode/nonstiff/AdamsBashforthFieldIntegratorTest.java b/src/test/java/org/apache/commons/math4/ode/nonstiff/AdamsBashforthFieldIntegratorTest.java
index 408e646..e9a046c 100644
--- a/src/test/java/org/apache/commons/math4/ode/nonstiff/AdamsBashforthFieldIntegratorTest.java
+++ b/src/test/java/org/apache/commons/math4/ode/nonstiff/AdamsBashforthFieldIntegratorTest.java
@@ -49,15 +49,15 @@ public class AdamsBashforthFieldIntegratorTest extends AbstractAdamsFieldIntegra
 
     @Test
     public void testIncreasingTolerance() {
         // the 7 and 121 factors are only valid for this test
         // and has been obtained from trial and error
         // there are no general relationship between local and global errors
-        doTestIncreasingTolerance(Decimal64Field.getInstance(), 7, 121);
+        doTestIncreasingTolerance(Decimal64Field.getInstance(), 2.6, 122);
     }
 
     @Test(expected = MaxCountExceededException.class)
     public void exceedMaxEvaluations() {
-        doExceedMaxEvaluations(Decimal64Field.getInstance());
+        doExceedMaxEvaluations(Decimal64Field.getInstance(), 650);
     }
 
     @Test

http://git-wip-us.apache.org/repos/asf/commons-math/blob/82cf2774/src/test/java/org/apache/commons/math4/ode/nonstiff/AdamsBashforthIntegratorTest.java
----------------------------------------------------------------------
@@ -49,15 +49,15 @@ public class AdamsBashforthFieldIntegratorTest extends AbstractAdamsFieldIntegra
     @Test
     public void testIncreasingTolerance() {
-        // the 7 and 121 factors are only valid for this test
+        // the 2.6 and 122 factors are only valid for this test
         // and has been obtained from trial and error
         // there are no general relationship between local and global errors